@@ -37,6 +37,9 @@
 #include <libgnomeui/libgnomeui.h>
 #include <libgnomevfs/gnome-vfs.h>
 
+#include <gtksourceview/gtksourcelanguage.h>
+#include <gtksourceview/gtksourcelanguagesmanager.h>
+
 #include <string.h>
 
 #include "gedit-mdi.h"
@@ -54,6 +57,7 @@
 #include "recent-files/egg-recent-view-bonobo.h"
 #include "recent-files/egg-recent-view-gtk.h"
 #include "recent-files/egg-recent-model.h"
+#include "gedit-languages-manager.h"
 
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-control.h>
@@ -76,8 +80,6 @@ struct _GeditWindowPrefs
 	GeditToolbarSetting toolbar_buttons_style;
 
 	gboolean statusbar_visible;
-	gboolean statusbar_show_cursor_position;
-	gboolean statusbar_show_overwrite_mode;
 
 	gboolean output_window_visible;
 };
@@ -104,6 +106,7 @@ static void gedit_mdi_child_changed_handler (BonoboMDI *mdi, BonoboMDIChild *old
 static void gedit_mdi_child_state_changed_handler (GeditMDIChild *child);
 
 static void gedit_mdi_set_active_window_undo_redo_verbs_sensitivity (BonoboMDI *mdi);
+static void gedit_mdi_set_active_window_find_verbs_sensitivity (BonoboMDI *mdi);
 
 static void gedit_mdi_app_destroy_handler (BonoboMDI *mdi, BonoboWindow *window);
 
@@ -113,7 +116,8 @@ static void gedit_mdi_view_menu_item_toggled_handler (
 			Bonobo_UIComponent_EventType type,
 			const char                  *state,
 			BonoboWindow                *win);
-
+static void add_languages_menu (BonoboMDI *mdi, BonoboWindow *win);
+static void gedit_mdi_update_languages_menu (BonoboMDI *mdi);
 
 static GQuark window_prefs_id = 0;
 
@@ -380,6 +384,7 @@ static void
 gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 {
 	GtkWidget *widget;
+
 	BonoboControl *control;
 	BonoboUIComponent *ui_component;
 	EggRecentView *view;
@@ -403,12 +408,12 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	g_signal_connect (G_OBJECT (win), "drag_data_received",
 			  G_CALLBACK (gedit_mdi_drag_data_received_handler), 
 			  NULL);
-	
+		
 	/* Add cursor position status bar */
 	widget = gtk_statusbar_new ();
 	control = bonobo_control_new (widget);
 	
-	gtk_widget_set_size_request (widget, 150, 10);	
+	gtk_widget_set_size_request (widget, 160, 10);	
 	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (widget), FALSE);
 	
 	bonobo_ui_component_object_set (ui_component,
@@ -416,6 +421,8 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 					BONOBO_OBJREF (control),
 					NULL);
 
+	gtk_widget_show (widget);
+
 	bonobo_object_unref (BONOBO_OBJECT (control));
 
 	g_object_set_data (G_OBJECT (win), "CursorPosition", widget);
@@ -431,6 +438,8 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 					BONOBO_OBJREF (control),
 					NULL);
 
+	gtk_widget_show (widget);
+
 	bonobo_object_unref (BONOBO_OBJECT (control));
 
 	g_object_set_data (G_OBJECT (win), "OverwriteMode", widget);
@@ -476,13 +485,6 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 	bonobo_ui_component_add_listener (ui_component, "ToolbarIconBothHoriz", 
 			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
 			(gpointer)win);
-
-	bonobo_ui_component_add_listener (ui_component, "StatusBarCursorPosition", 
-			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
-			(gpointer)win);
-	bonobo_ui_component_add_listener (ui_component, "StatusBarOverwriteMode", 
-			(BonoboUIListenerFn)gedit_mdi_view_menu_item_toggled_handler, 
-			(gpointer)win);
 	
 	/* add a GeditRecentView object */
 	model = gedit_recent_get_model ();
@@ -525,6 +527,8 @@ gedit_mdi_app_created_handler (BonoboMDI *mdi, BonoboWindow *win)
 		else
 			gtk_window_unstick (GTK_WINDOW (win));
 	}
+
+	add_languages_menu (mdi, win);
 	
 	/* Add the plugins menus */
 	gedit_plugins_engine_update_plugins_ui (win, TRUE);
@@ -646,28 +650,6 @@ gedit_mdi_view_menu_item_toggled_handler (
 		goto save_prefs;
 	}
 
-	if (strcmp (path, "StatusBarCursorPosition") == 0)
-	{
-		if (s != prefs->statusbar_show_cursor_position)
-		{
-			prefs->statusbar_show_cursor_position = s;
-			gedit_mdi_set_app_statusbar_style (win);
-		}
-
-		goto save_prefs;
-	}
-
-	if (strcmp (path, "StatusBarOverwriteMode") == 0)
-	{
-		if (s != prefs->statusbar_show_overwrite_mode)
-		{
-			prefs->statusbar_show_overwrite_mode = s;
-			gedit_mdi_set_app_statusbar_style (win);
-		}
-
-		goto save_prefs;
-	}
-
 save_prefs:
 	gedit_window_prefs_save (prefs);
 }
@@ -813,7 +795,6 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 {
 	GeditWindowPrefs *prefs = NULL;
 	BonoboUIComponent *ui_component;
-	GtkWidget *cp, *om;
 	
 	gedit_debug (DEBUG_MDI, "");
 	
@@ -831,22 +812,6 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 	gedit_menus_set_verb_state (ui_component, 
 				    "/commands/ViewStatusbar",
 				    prefs->statusbar_visible);
-
-	gedit_menus_set_verb_sensitive (ui_component, 
-				    "/commands/StatusBarCursorPosition",
-				    prefs->statusbar_visible);
-
-	gedit_menus_set_verb_sensitive (ui_component, 
-				    "/commands/StatusBarOverwriteMode",
-				    prefs->statusbar_visible);
-
-	gedit_menus_set_verb_state (ui_component, 
-				    "/commands/StatusBarCursorPosition",
-				    prefs->statusbar_show_cursor_position);
-
-	gedit_menus_set_verb_state (ui_component, 
-				    "/commands/StatusBarOverwriteMode",
-				    prefs->statusbar_show_overwrite_mode);
 	
 	/* Actually update status bar style */
 	bonobo_ui_component_set_prop (
@@ -854,60 +819,7 @@ gedit_mdi_set_app_statusbar_style (BonoboWindow *win)
 		"hidden", prefs->statusbar_visible ? "0" : "1",
 		NULL);
 
-	cp = GTK_WIDGET (g_object_get_data (G_OBJECT (win), "CursorPosition"));
-	if (cp == NULL)
-		goto error;
-
-	if (prefs->statusbar_show_cursor_position)
-	{
-		bonobo_ui_component_set_prop (
-			ui_component, "/status/CursorPosition", "hidden", "0", NULL);
-
-		gtk_widget_show (cp);
-	}
-	else
-	{
-		bonobo_ui_component_set_prop (
-			ui_component, "/status/CursorPosition", "hidden", "1", NULL);
-
-		gtk_widget_hide (cp);
-	}
-	
-	om = GTK_WIDGET (g_object_get_data (G_OBJECT (win), "OverwriteMode"));
-	if (om == NULL)
-		goto error;
-
-	if (prefs->statusbar_show_overwrite_mode)
-	{
-		gtk_widget_show (om);
-		
-		bonobo_ui_component_set_prop (
-			ui_component, "/status/OverwriteMode", "hidden", "0", NULL);
-	}
-	else
-	{
-		bonobo_ui_component_set_prop (
-			ui_component, "/status/OverwriteMode", "hidden", "1", NULL);
-		
-		gtk_widget_hide (om);
-	}
-
-	if (!prefs->statusbar_show_overwrite_mode)
-		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (cp), TRUE);
-	else
-		gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (cp), FALSE);
-
-	if (!prefs->statusbar_show_cursor_position &&
-	    !prefs->statusbar_show_overwrite_mode)
-		bonobo_ui_component_set_prop (
-			ui_component, "/status", "resize_grip", "1", NULL);
-	else
-		bonobo_ui_component_set_prop (
-			ui_component, "/status", "resize_grip", "0", NULL);
-
-error:
 	bonobo_ui_component_thaw (ui_component, NULL);
-	
 }
 
 static void 
@@ -920,6 +832,7 @@ gedit_mdi_child_state_changed_handler (GeditMDIChild *child)
 	
 	gedit_mdi_set_active_window_title (BONOBO_MDI (gedit_mdi));
 	gedit_mdi_set_active_window_verbs_sensitivity (BONOBO_MDI (gedit_mdi));
+	gedit_mdi_update_languages_menu (BONOBO_MDI (gedit_mdi));
 }
 
 static void 
@@ -933,6 +846,17 @@ gedit_mdi_child_undo_redo_state_changed_handler (GeditMDIChild *child)
 	gedit_mdi_set_active_window_undo_redo_verbs_sensitivity (BONOBO_MDI (gedit_mdi));
 }
 
+static void 
+gedit_mdi_child_find_state_changed_handler (GeditMDIChild *child)
+{
+	gedit_debug (DEBUG_MDI, "");
+
+	if (bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi)) != BONOBO_MDI_CHILD (child))
+		return;
+	
+	gedit_mdi_set_active_window_find_verbs_sensitivity (BONOBO_MDI (gedit_mdi));
+}
+
 static gint 
 gedit_mdi_add_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 {
@@ -944,6 +868,9 @@ gedit_mdi_add_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 	g_signal_connect (G_OBJECT (child), "undo_redo_state_changed",
 			  G_CALLBACK (gedit_mdi_child_undo_redo_state_changed_handler), 
 			  NULL);
+	g_signal_connect (G_OBJECT (child), "find_state_changed",
+			  G_CALLBACK (gedit_mdi_child_find_state_changed_handler), 
+			  NULL);
 
 	return TRUE;
 }
@@ -1104,6 +1031,9 @@ gedit_mdi_remove_child_handler (BonoboMDI *mdi, BonoboMDIChild *child)
 		g_signal_handlers_disconnect_by_func (GTK_OBJECT (child), 
 						      G_CALLBACK (gedit_mdi_child_undo_redo_state_changed_handler),
 						      NULL);
+		g_signal_handlers_disconnect_by_func (child, 
+						      G_CALLBACK (gedit_mdi_child_find_state_changed_handler),
+						      NULL);
 	}
 	*/
 	
@@ -1167,8 +1097,9 @@ static
 void gedit_mdi_child_changed_handler (BonoboMDI *mdi, BonoboMDIChild *old_child)
 {
 	gedit_debug (DEBUG_MDI, "");
-
+		
 	gedit_mdi_set_active_window_title (mdi);	
+	gedit_mdi_update_languages_menu (mdi);
 }
 
 static 
@@ -1241,7 +1172,6 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	BonoboMDIChild* active_child = NULL;
 	GeditDocument* doc = NULL;
 	BonoboUIComponent *ui_component;
-	gchar *lst;
 	
 	gedit_debug (DEBUG_MDI, "");
 	
@@ -1263,12 +1193,20 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	{
 		gedit_menus_set_verb_list_sensitive (ui_component, 
 				gedit_menus_no_docs_sensible_verbs, FALSE);
+
+		bonobo_ui_component_set_prop (
+			ui_component, "/menu/View/HighlightMode", "sensitive", "0", NULL);
+
 		goto end;
 	}
 	else
 	{
 		gedit_menus_set_verb_list_sensitive (ui_component, 
 				gedit_menus_all_sensible_verbs, TRUE);
+
+		bonobo_ui_component_set_prop (
+			ui_component, "/menu/View/HighlightMode", "sensitive", "1", NULL);
+
 	}
 
 	gedit_menus_set_verb_sensitive (ui_component, "/commands/DocumentsMoveToNewWindow",
@@ -1277,10 +1215,11 @@ gedit_mdi_set_active_window_verbs_sensitivity (BonoboMDI *mdi)
 	doc = GEDIT_MDI_CHILD (active_child)->document;
 	g_return_if_fail (doc != NULL);
 	
-	lst = gedit_document_get_last_searched_text (doc);
-
-	gedit_menus_set_verb_sensitive (ui_component, "/commands/SearchFindAgain", lst != NULL);	
-	g_free (lst);
+	if (!gedit_document_can_find_again (doc))
+	{
+		gedit_menus_set_verb_sensitive (ui_component, "/commands/SearchFindNext", FALSE);
+		gedit_menus_set_verb_sensitive (ui_component, "/commands/SearchFindPrev", FALSE);
+	}
 		
 	if (gedit_document_is_readonly (doc))
 	{
@@ -1317,20 +1256,18 @@ static void
 gedit_mdi_set_active_window_undo_redo_verbs_sensitivity (BonoboMDI *mdi)
 {
 	BonoboWindow* active_window = NULL;
-	BonoboMDIChild* active_child = NULL;
 	GeditDocument* doc = NULL;
 	BonoboUIComponent *ui_component;
 	
 	gedit_debug (DEBUG_MDI, "");
 	
-	active_window = bonobo_mdi_get_active_window (mdi);
+	active_window = gedit_get_active_window ();
 	g_return_if_fail (active_window != NULL);
 	
-	ui_component = bonobo_mdi_get_ui_component_from_window (active_window);
+	ui_component = gedit_get_ui_component_from_window (active_window);
 	g_return_if_fail (ui_component != NULL);
 	
-	active_child = bonobo_mdi_get_active_child (mdi);
-	doc = GEDIT_MDI_CHILD (active_child)->document;
+	doc = gedit_get_active_document ();
 	g_return_if_fail (doc != NULL);
 
 	bonobo_ui_component_freeze (ui_component, NULL);
@@ -1344,6 +1281,39 @@ gedit_mdi_set_active_window_undo_redo_verbs_sensitivity (BonoboMDI *mdi)
 	bonobo_ui_component_thaw (ui_component, NULL);
 }
 
+static void 
+gedit_mdi_set_active_window_find_verbs_sensitivity (BonoboMDI *mdi)
+{
+	BonoboWindow* active_window = NULL;
+	GeditDocument* doc = NULL;
+	BonoboUIComponent *ui_component;
+	gboolean sensitive;
+	
+	gedit_debug (DEBUG_MDI, "");
+	
+	active_window = gedit_get_active_window ();
+	g_return_if_fail (active_window != NULL);
+	
+	ui_component = gedit_get_ui_component_from_window (active_window);
+	g_return_if_fail (ui_component != NULL);
+	
+	doc = gedit_get_active_document ();
+	g_return_if_fail (doc != NULL);
+
+	sensitive = gedit_document_can_find_again (doc);
+
+	bonobo_ui_component_freeze (ui_component, NULL);
+
+	gedit_menus_set_verb_sensitive (ui_component, "/commands/SearchFindNext",
+					sensitive);
+
+	gedit_menus_set_verb_sensitive (ui_component, "/commands/SearchFindPrev",
+					sensitive);
+
+	bonobo_ui_component_thaw (ui_component, NULL);
+}
+
+
 EggRecentView *
 gedit_mdi_get_recent_view_from_window (BonoboWindow *win)
 {
@@ -1461,8 +1431,6 @@ gedit_window_prefs_new (void)
 	prefs->toolbar_buttons_style = gedit_prefs_manager_get_toolbar_buttons_style ();
 
 	prefs->statusbar_visible = gedit_prefs_manager_get_statusbar_visible ();
-	prefs->statusbar_show_cursor_position = gedit_prefs_manager_get_statusbar_show_cursor_position ();
-	prefs->statusbar_show_overwrite_mode = gedit_prefs_manager_get_statusbar_show_overwrite_mode ();
 
 	prefs->output_window_visible = FALSE;
 
@@ -1518,15 +1486,324 @@ gedit_window_prefs_save (GeditWindowPrefs *prefs)
 	    gedit_prefs_manager_statusbar_visible_can_set ())
 		gedit_prefs_manager_set_statusbar_visible (prefs->statusbar_visible);
 
-	if ((prefs->statusbar_show_cursor_position != 
-			gedit_prefs_manager_get_statusbar_show_cursor_position ()) &&
-	    gedit_prefs_manager_statusbar_show_cursor_position_can_set ())
-		gedit_prefs_manager_set_statusbar_show_cursor_position (
-				prefs->statusbar_show_cursor_position);
-
-	if ((prefs->statusbar_show_overwrite_mode != 
-			gedit_prefs_manager_get_statusbar_show_overwrite_mode ()) &&
-	    gedit_prefs_manager_statusbar_show_overwrite_mode_can_set ())
-		gedit_prefs_manager_set_statusbar_show_overwrite_mode (
-				prefs->statusbar_show_overwrite_mode);
 }
+
+static gchar* 
+escape_underscores (const gchar* text)
+{
+	GString *str;
+	gint length;
+	const gchar *p;
+ 	const gchar *end;
+
+  	g_return_val_if_fail (text != NULL, NULL);
+
+    	length = strlen (text);
+
+	str = g_string_new ("");
+
+  	p = text;
+  	end = text + length;
+
+  	while (p != end)
+    	{
+      		const gchar *next;
+      		next = g_utf8_next_char (p);
+
+		switch (*p)
+        	{
+       			case '_':
+          			g_string_append (str, "__");
+          			break;
+        		default:
+          			g_string_append_len (str, p, next - p);
+          			break;
+        	}
+
+      		p = next;
+    	}
+
+	return g_string_free (str, FALSE);
+}
+
+static gchar* 
+replace_spaces_with_underscores (const gchar* text)
+{
+	GString *str;
+	gint length;
+	const gchar *p;
+ 	const gchar *end;
+
+  	g_return_val_if_fail (text != NULL, NULL);
+
+    	length = strlen (text);
+
+	str = g_string_new ("");
+
+  	p = text;
+  	end = text + length;
+
+  	while (p != end)
+    	{
+      		const gchar *next;
+      		next = g_utf8_next_char (p);
+
+		switch (*p)
+        	{
+       			case ' ':
+			case '\t':
+			case '+':
+          			g_string_append (str, "_");
+          			break;
+        		default:
+          			g_string_append_len (str, p, next - p);
+          			break;
+        	}
+
+      		p = next;
+    	}
+
+	return g_string_free (str, FALSE);
+}
+
+static void
+slist_deep_free (GSList *list)
+{
+	g_slist_foreach (list, (GFunc) g_free, NULL);
+	g_slist_free (list);
+}
+	
+static gboolean
+section_exists (GSList      *list,
+		const gchar *section)
+{
+	while (list != NULL)
+	{
+      		if (strcmp (list->data, section) == 0)
+			return TRUE;
+
+		list = g_slist_next (list);
+    	}
+
+  	return FALSE;
+}
+
+static gchar *
+get_base_path_for_language (GtkSourceLanguage *lang)
+{
+	gchar *safe_name;
+	gchar *section;
+	gchar *name;
+	gchar *path;
+
+	section = gtk_source_language_get_section (lang);
+	
+	safe_name = g_markup_escape_text (section, strlen (section));
+	name = replace_spaces_with_underscores (safe_name);
+		
+	path = g_strdup_printf ("/menu/View/HighlightMode/%s/", name);
+
+	g_free (safe_name);
+	g_free (name);
+	g_free (section);
+
+	return path;
+}
+	
+static void
+language_toggled_handler (
+			BonoboUIComponent           *ui_component,
+			const char                  *path,
+			Bonobo_UIComponent_EventType type,
+			const char                  *state,
+			GtkSourceLanguage           *lang)
+{
+	GeditDocument *doc;
+	gboolean s;
+
+       	s = (strcmp (state, "1") == 0);
+	
+	if (!s)
+		return;
+	
+	doc = gedit_get_active_document ();
+	g_return_if_fail (doc != NULL);
+
+	gedit_document_set_language (doc, lang);
+}
+
+
+static gchar *
+get_verb_name_for_language (GtkSourceLanguage *lang)
+{
+	gchar *lang_name;
+	gchar *safe_name;
+	gchar *name;
+	gchar *verb_name;
+
+	if (lang == NULL)
+		return g_strdup ("/commands/gedit_lang__Normal");
+
+	lang_name = gtk_source_language_get_name (lang);
+	safe_name = g_markup_escape_text (lang_name, strlen (lang_name));
+	
+	name = replace_spaces_with_underscores (safe_name);
+
+	verb_name = g_strdup_printf ("/commands/gedit_lang__%s", name);
+		
+	g_free (safe_name);
+	g_free (lang_name);
+	g_free (name);
+
+	return verb_name;
+}
+
+
+static void
+add_languages_menu (BonoboMDI *mdi, BonoboWindow *win)
+{
+	const GSList *languages;
+	GSList *sections = NULL;
+	const GSList *l;
+	GSList *s;
+
+	gedit_debug (DEBUG_MDI, "");
+
+	gedit_menus_add_menu_item_radio (win,
+					 "/menu/View/HighlightMode/",
+					 "gedit_lang__Normal",
+					 "Langs",
+					 _("Normal"),
+					 _("Use Normal highlight mode"),
+					 (BonoboUIListenerFn)language_toggled_handler,
+					 NULL);
+					  
+	languages = gtk_source_languages_manager_get_available_languages (
+						gedit_get_languages_manager ());
+
+	l = languages;
+
+	while (l != NULL)
+	{
+		gchar *section;
+				
+		section = gtk_source_language_get_section (GTK_SOURCE_LANGUAGE (l->data));
+
+		if (!section_exists (sections, section))
+			sections = g_slist_prepend (sections, section);
+		else
+			g_free (section);
+
+		l = g_slist_next (l);
+	}
+
+	/*
+	sections = g_slist_sort (sections, (GCompareFunc)strcmp);
+	*/
+	sections = g_slist_reverse (sections);
+
+	s = sections;
+
+	while (s != NULL)
+	{
+		gchar *safe_name;
+		gchar *label;
+		gchar *name;
+
+		safe_name = g_markup_escape_text (s->data, strlen (s->data));
+		label = escape_underscores (safe_name);
+		name = replace_spaces_with_underscores (safe_name);
+		
+		gedit_menus_add_submenu (win, 
+					 "/menu/View/HighlightMode/",
+					 name,
+					 label);
+	
+		g_free (safe_name);
+		g_free (label);
+		g_free (name);
+		
+		s = g_slist_next (s);
+	}
+
+	slist_deep_free (sections);
+
+	l = languages;
+	
+	while (l != NULL)
+	{
+		gchar *path;
+		
+		gchar *lang_name;
+		gchar *safe_name;
+		gchar *label;
+		gchar *name;
+		gchar *tip;
+		gchar *tmp;
+
+		path = get_base_path_for_language (l->data);
+
+		lang_name = gtk_source_language_get_name (l->data);
+		safe_name = g_markup_escape_text (lang_name, strlen (lang_name));
+		label = escape_underscores (safe_name);
+		tmp = replace_spaces_with_underscores (safe_name);
+		name = g_strdup_printf ("gedit_lang__%s", tmp);
+		g_free (tmp);
+		
+		tip = g_strdup_printf (N_("Use %s highlight mode"), safe_name);
+
+		gedit_menus_add_menu_item_radio (win,
+					  path,
+					  name,
+					  "Langs",
+					  label,
+					  tip,
+					  (BonoboUIListenerFn)language_toggled_handler,
+					  l->data);
+
+		g_free (safe_name);
+		g_free (label);
+		g_free (name);
+		g_free (tip);
+		g_free (lang_name);
+		g_free (path);
+
+		l = g_slist_next (l);
+	}		
+}
+
+static 
+void gedit_mdi_update_languages_menu (BonoboMDI *mdi)
+{
+	GtkSourceLanguage *lang;
+	BonoboMDIChild *active_child;
+	GeditDocument *doc;
+	gchar *verb_name;
+	BonoboWindow* active_window = NULL;
+	BonoboUIComponent *ui_component;
+	
+	gedit_debug (DEBUG_MDI, "");
+		
+	active_window = bonobo_mdi_get_active_window (mdi);
+
+	if (active_window == NULL)
+		return;
+	
+	ui_component = bonobo_mdi_get_ui_component_from_window (active_window);
+	g_return_if_fail (ui_component != NULL);
+
+	active_child = bonobo_mdi_get_active_child (mdi);
+	if (active_child == NULL)
+		return;
+
+	doc = GEDIT_MDI_CHILD (active_child)->document;
+	g_return_if_fail (doc != NULL);
+
+	lang = gedit_document_get_language (doc);
+	verb_name = get_verb_name_for_language (lang);
+	
+	gedit_menus_set_verb_state (ui_component, verb_name, TRUE);
+
+	g_free (verb_name);
+}
+
